@@ -16,7 +16,6 @@ Trading project:
 #   numpy
 #   scalper and dca bot
 
-import random
 import time
 import typing
 from typing import Dict, List, Tuple, Any, Iterable
@@ -152,7 +151,6 @@ class Trader(object):
             self._sec_interval = 86400 * 180
         else:
             raise ValueError(f'incorrect interval; {interval}')
-        self._regression_inputs = utils.REGRESSION_INPUTS
         self.__exit_order__ = False
 
     def __repr__(self):
@@ -205,19 +203,6 @@ class Trader(object):
                     line=dict(width=utils.SUB_LINES_WIDTH)), 1, 1)
         return pd.Series(filtered)
 
-    def bull_power(self, periods: int) -> np.ndarray:
-        EMA = ta.trend.ema_indicator(self.df['Close'], periods)
-        return np.array(self.df['High']) - EMA
-
-    def tema(self, periods: int, *args, **kwargs) -> pd.Series:
-        """
-        :rtype: pd.Series
-        """
-        ema = ta.trend.ema_indicator(self.df['Close'], periods)
-        ema2 = ta.trend.ema_indicator(ema, periods)
-        ema3 = ta.trend.ema_indicator(ema2, periods)
-        return pd.Series(3 * ema.values - 3 * ema2.values + ema3.values)
-
     def __get_stop_take(self, sig: utils.PREDICT_TYPE) -> Dict[str, float]:
         """
         calculating stop loss and take profit.
@@ -296,6 +281,7 @@ class Trader(object):
 
     def strategy_buy_hold(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
         self.returns = [utils.BUY for _ in range(len(self.df))]
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -328,6 +314,7 @@ class Trader(object):
             else:
                 self.returns.append(utils.EXIT)
         self.set_open_stop_and_take()
+        self.set_credit_leverages()
         return self.returns
 
     def strategy_3_sma(self,
@@ -360,6 +347,7 @@ class Trader(object):
             else:
                 self.returns.append(utils.EXIT)
 
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -391,6 +379,7 @@ class Trader(object):
                 self.returns.append(utils.SELL)
             else:
                 self.returns.append(utils.EXIT)
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -409,23 +398,7 @@ class Trader(object):
                 self.returns.append(utils.SELL)
             else:
                 self.returns.append(utils.EXIT)
-        self.set_open_stop_and_take()
-        return self.returns
-
-    def strategy_exp_diff(self,
-                          period: int = 70,
-                          plot: bool = True,
-                          *args,
-                          **kwargs) -> utils.PREDICT_TYPE_LIST:
-        exp: pd.Series = self.tema(period)
-        self.strategy_diff(exp)
-        if plot:
-            self.fig.add_trace(
-                Line(
-                    name=f'EMA{period}',
-                    y=exp.values.T[0],
-                    line=dict(width=utils.SUB_LINES_WIDTH)), 1, 1)
-
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -451,6 +424,7 @@ class Trader(object):
                 flag = utils.EXIT
             self.returns.append(flag)
 
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -478,7 +452,8 @@ class Trader(object):
                 self.returns.append(utils.SELL)
             else:
                 self.returns.append(utils.EXIT)
-        self.set_open_stop_and_take(set_stop=False)
+        self.set_credit_leverages()
+        self.set_open_stop_and_take()
         return self.returns
 
     def strategy_macd_histogram_diff(self,
@@ -497,11 +472,7 @@ class Trader(object):
                 self.returns.append(utils.BUY)
             else:
                 self.returns.append(utils.SELL)
-        self.set_open_stop_and_take()
-        return self.returns
-
-    def strategy_random_pred(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
-        self.returns = [random.choice([utils.EXIT, utils.SELL, utils.BUY]) for i in range(len(self.df))]
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -522,6 +493,7 @@ class Trader(object):
         self.returns = list(st.get_supertrend_strategy_returns())
         self._stop_losses[0] = np.inf if self.returns[0] == utils.SELL else -np.inf
         self.set_open_stop_and_take(set_stop=False)
+        self.set_credit_leverages()
         return self.returns
 
     def strategy_bollinger(self,
@@ -686,7 +658,7 @@ class Trader(object):
                     self._stop_losses.append(max_cloud + stop_loss_adder)
         self.set_open_stop_and_take(set_take=True,
                                     set_stop=False)
-        self.set_open_stop_and_take()
+        self.set_credit_leverages()
         return self.returns
 
     def crossover(self, fast: Iterable, slow: Iterable):
@@ -728,7 +700,7 @@ class Trader(object):
     def backtest(self,
                  deposit: float = 10_000.0,
                  bet: float = np.inf,
-                 commission: float = 0.0,
+                 commission_reuse: float = 0.0,
                  plot: bool = True,
                  print_out: bool = True,
                  column: str = 'Close',
@@ -739,7 +711,7 @@ class Trader(object):
         testing the strategy.
         :param deposit: start deposit.
         :param bet: fixed bet to quick_trade. np.inf = all moneys.
-        :param commission: percentage commission (0 -- 100).
+        :param commission_reuse: percentage commission (0 -- 100).
         :param plot: plotting.
         :param print_out: printing.
         :param column: column of dataframe to backtest
@@ -770,7 +742,7 @@ class Trader(object):
         moneys_open_bet: float = deposit
         money_start: float = deposit
         oldsig = utils.EXIT
-        start_commission: float = commission
+        start_commission: float = commission_reuse
 
         e: int
         sig: utils.PREDICT_TYPE
@@ -795,15 +767,16 @@ class Trader(object):
 
             if seted is not np.nan:
                 if oldsig != utils.EXIT:
-                    commission = start_commission * 2
+                    commission_reuse = 2
                 else:
-                    commission = start_commission
+                    commission_reuse = 1
                 if bet > deposit:
                     bet = deposit
                 open_price = data_column[e]
-                deposit -= bet * (commission / 100) * credit_lev
-                if bet > deposit:
-                    bet = deposit
+                for i in range(commission_reuse):
+                    deposit -= bet * (start_commission / 100) * credit_lev
+                    if bet > deposit:
+                        bet = deposit
                 moneys_open_bet = deposit
                 no_order = False
                 exit_take_stop = False
@@ -969,7 +942,7 @@ winrate: {self.winrate}%"""
         return self.backtest_out
 
     def multi_backtest(self,
-                       tickers: List[str],
+                       tickers: Iterable[str],
                        strategy_name: str = 'strategy_macd',
                        strategy_kwargs: Dict[str, Any] = {},
                        deposit: float = 10_000.0,
@@ -995,7 +968,7 @@ winrate: {self.winrate}%"""
             new_trader._get_attr(strategy_name)(**strategy_kwargs)
             new_trader.backtest(deposit=deposit / len(tickers),
                                 bet=bet,
-                                commission=commission,
+                                commission_reuse=commission,
                                 plot=False,
                                 print_out=False,
                                 column=column,
@@ -1465,6 +1438,7 @@ winrate: {self.winrate}%"""
                 self.returns.append(flag)
             else:
                 self.returns.append(flag)
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -1494,7 +1468,7 @@ winrate: {self.winrate}%"""
 
             self.returns.append(flag)
             self._stop_losses.append(flag_stop_loss)
-        self.set_open_stop_and_take(set_take=False, set_stop=False)
+        self.set_credit_leverages()
         return self.returns
 
     def find_TBH_TBL(self, *args, **kwargs) -> utils.PREDICT_TYPE_LIST:
@@ -1515,6 +1489,7 @@ winrate: {self.winrate}%"""
             elif low[0] == low[1]:
                 flag = utils.SELL
             self.returns.append(flag)
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -1535,6 +1510,7 @@ winrate: {self.winrate}%"""
                      ) == high[1] and close[2] < close[1] and low[2] > low[0]:
                 flag = utils.SELL
             self.returns.append(flag)
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return self.returns
 
@@ -1549,6 +1525,7 @@ winrate: {self.winrate}%"""
                 ret.append(True)
             else:
                 ret.append(False)
+        self.set_credit_leverages()
         self.set_open_stop_and_take()
         return ret
 
